/*
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
	FFNL32 1.1  19/7/99

   Multi-pattern Find Filename & Len

	Usage: FFNL32 [options] [d:][path]filename [&|!] [filename] ...

	options:

	-C[hh] set separator char (default: none)
    -D search only directory name (default files & dir)
    -F search only filename (default files & dir)
    -M[n] set max element size (default -1)
    -R recourse subdirectories


   Questa versione e' compilabile con:
   -	Borland C++ 4.X & STL HP (self modificate per funzionare senza warning e duplicazioni di funzioni)
   -    Borland C++ 5.X e standard C++ library fornite direttamente con il compilatore
   -    Borland C++ Builder 3 e standard C++ library fornite direttamente con il compilatore
   -    Microsoft Visual C++ 4.X e standard C++ library fornite dal compilatore
   -    Microsoft Visual C++ 5.X e standard C++ library fornite dal compilatore
   -    Microsoft Visual C++ 6.X e standard C++ library fornite dal compilatore

   Le condizioni in cui ho sviluppato il programma:

	BC++ 4.53 su Windows for Workgroup 3.11 & Win32s 1.30.166
	BC++ 5.x & VC++ 4.x su Windows NT 4.0
	BC++ 5.x & VC++ 5 SP3 su Windows NT 4.0 SP3

    Marco Savegnago (msave@tin.it)

	Luglio 1999
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

//	RTL include

#ifdef	__BORLANDC__
#include	<dir.h>
#include	<iostream.h>
#include	<iomanip.h>
#include	<fstream.h>
#endif

#ifdef	_MSC_VER
#include	<direct.h>
#include	<windows.h>
#include	<iostream>
#include	<iomanip>
#include	<fstream>
#endif

#include	<stdio.h>
#include	<ctype.h>
#include	<dos.h>
#include	<io.h>
#include	<errno.h>
#include	<conio.h>
#include	<stdlib.h>

//	STL include

#if defined(__BORLANDC__)
	#if 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53

	#include	<cstring.h>
	#include	<stl\list.h>
	#include	<stl\algo.h>

	typedef	list<string>					stringList;
	typedef	list<string>::iterator		stringListIterator;

	#else

	#include	<list>
	#include	<string>

	using namespace std;

	typedef	list<string>					stringList;
	typedef	list<string>::iterator		stringListIterator;

	#endif

#elif defined (_MSC_VER)		//	VC++ & BC++ accettano gli header files senza estensione
#include	<string>
#include	<list>
#include	<algorithm>

using namespace std;

typedef	allocator<string>					stringAllocator;
typedef	list<string, stringAllocator>	stringList;
typedef	list<string>::iterator			stringListIterator;

#endif

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Classe usata dagli include/exclude iterator per per il filename matching
 *       con i wildcard standard dos (*?)
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

class	matchFilename
{
public:
	char 	sz_drive[_MAX_DRIVE];
	char 	sz_dir[_MAX_DIR];
	char 	sz_filename[_MAX_FNAME];
	char	sz_extname[_MAX_EXT];
	char	sz_filePattern[_MAX_FNAME];
	char	sz_extPattern[_MAX_EXT];

	matchFilename		(const char*	cpsz_fname);

#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
	bool	operator() 	(string*	psc_pattern);
#else
	bool	operator() 	(string		sc_pattern);
#endif

	bool	match_fname (char *csz_fname, char *csz_pattern);
};

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: matchFilename ctor predefinisce i componenti da usare per il matching
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

matchFilename::matchFilename(const char*	cpsz_fname)
{
	_splitpath(cpsz_fname,sz_drive,sz_dir,sz_filename,sz_extname);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: esegue il filename matching separando il nome dall'estensione
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
bool	matchFilename::operator() (string*	psc_pattern)
{
	_splitpath(psc_pattern.c_str(),sz_drive,sz_dir,sz_filePattern,sz_extPattern);
	return (match_fname(sz_filename,sz_filePattern) & match_fname(sz_extname,sz_extPattern));
}
#else
bool	matchFilename::operator() (string	sc_pattern)
{
	_splitpath(sc_pattern.c_str(),sz_drive,sz_dir,sz_filePattern,sz_extPattern);
	return (match_fname(sz_filename,sz_filePattern) & match_fname(sz_extname,sz_extPattern));
}
#endif

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: esegue il matching tra elemento e pattern
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

bool	matchFilename::match_fname (char *psz_fname, char *psz_pattern)
{
	register char*	p;
	register char*	f;
	bool 			b_match=true;
	bool			b_skip=false;

	if (*psz_pattern=='*')
		return true;

	if ( strpbrk(psz_pattern, "*?") )
	{
   	if (!strchr(psz_pattern,'*') && (strlen(psz_pattern) != strlen(psz_fname)))
      		return false;

		p=psz_pattern;
		f=psz_fname;

		while(*f && *p && !b_skip)
		{
      		switch (*p)
	    		{
	    			case '*':
               {
						b_skip=true;
						break;
               }
	    			case '?':
						break;

	    			default:
               {
						if(toupper(*f) != toupper(*p))
						{
		    				b_match=false;
		    				b_skip=true;
						}
						break;
               }
	    		}
	    		p++;
				f++;
      }
	}
	else
		if (stricmp(psz_fname,psz_pattern))
      		b_match=false;

   return (b_match);
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Classe usata dagli include/exclude iterator per per il filename matching
 *       con i wildcard estesi (vedi note sotto)
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

class	matchDirname
{
	bool isAWildCardIncluded(const char* s);

public:
	char 	sz_dir[_MAX_PATH];

	matchDirname	(const char*	cpsz_fname);

#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
	bool	operator() 	(string*	psc_pattern);
#else
	bool	operator() 	(string		sc_pattern);
#endif

	bool	match_fname (const	char *csz_fname, const char *csz_pattern);
};

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: matchDirname ctor predefinisce i componenti da usare per il matching
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

matchDirname::matchDirname(const char*	cpsz_fname)
{
	strcpy(sz_dir, cpsz_fname);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: esegue il directory name matching
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
bool	matchDirname::operator() (string*	psc_pattern)
{
	return (match_fname(sz_dir, psc_pattern));
}
#else
bool	matchDirname::operator() (string	sc_pattern)
{
	return (match_fname(sz_dir, sc_pattern.c_str()));
}
#endif

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: ritorna true se la strinha contiene wildcard
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

bool matchDirname::isAWildCardIncluded(const char* s)
{
	return bool ( strpbrk ( s, "*@#+?<>$\"()") != NULL );
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Questa funzione compara 2 stringe usando il pattern matching definito secondo le regole che seguono
 
	The pattern can contain this wildcard:

	Special characters in the pattern are:

     * -- Matches 0 or more characters of any type
     @ -- a..z, A..Z
     # -- 0..9
	  + -- A..z, A..Z, 0..9
     ? -- A-F, a-f, 0-9 (a hex number)
	 < -- a..z
	 > -- A..Z
     = -- Any one character
     " -- Escape.  The next character in the pattern much match exactly.
			 wild cards will not be interpreted.
     ( -- Group.  A "(" starts a group and a ")" ends it.  For each
          group, there must be a matching character in the string.
			 Example: (abc) matches b.  Wild cards are not interpreted.
     \ -- The string to the right of the \ is optional.  It can be
          present and if so must match.  If it is missing, it is
          a match.  Example: a\a will match with a or aa but not ab

Examples:

  36*      --  Will match anything starting with 36 such as 36020,
               36123, 36asb, and 36.
  36===    --  Will match anything that starts with 36 and is 5 characters
               long.  36020 will match but 3602 will not.
  36###    --  Will match anything that starts with 36 and has 5 digits.
               36020 will match but 36ABC will not.
  (IK)3*   --  Will match anything starting with I3 or K3.
  #VI\.ITA\.EU\.EURO -- Will match with #VI.EU or #VI but not with
                         #VI.XXXX

 */


bool	matchDirname::match_fname (const	char *psz_fname, const	char *psz_pattern)
{
	const	bool usePattern		=	true;
	register const char *f;
	register const char *p;

	bool match = true;
	bool skip = false;

	if ( usePattern && isAWildCardIncluded( psz_pattern ) )
	{
		p = psz_pattern;
		f = psz_fname;

		while( ! skip )
		{
			switch ( *p )
			{
         	case	'\0':
            	skip=true;

					if(*f)
               	match=false;

					break;

            case '\\':
				{
            	while(*p && strlen(p) > 1)
               {
               	if(match_fname( f, ++p))
                  	return	match = true;

                  while(*p && *p != '\\')
                  	p++;
               }
               break;
            }

				case '*':
            	if(strlen(p) > 1)
					{
               	p++;

						bool	lmatch = false;

               	while(*(++f))
						{
							if(*p == *f)
							{
                     	lmatch	=	true;
								break;
							}
						}

						if(lmatch && *f && match_fname( f, p))
							return	match = true;
					}
					else
						return	match = true;

					break;
				case '@':
					if( ! isalpha( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '#':
					if( ! isdigit ( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '+':
					if( ! isalnum ( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '?':
            		if( ! isxdigit( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '<':
					if( ! islower( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '>':
					if( ! isupper( *f) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;

				case '(':
				{
					bool lmatch=false;

					while (*++p != ')' )
					{
						if( toupper( *f ) == toupper( *p ) )
									lmatch = true;
					}

					if (!lmatch)
					{
						match=false;
						skip=true;
					}
					p++; f++;
					break;
				}

				case '=':
            		p++;

					if(*f)
						f++;
					else
					{
               			match=false;
						skip = true;
					}

					break;

				case '"':
					p++;
				default:
					if( toupper ( *f ) != toupper( *p ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
			}
		}
	}
	else
	{
   	if ( strcmp  ( psz_fname, psz_pattern) )
								match = false;
	}

	return ( match );
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Classe che defisce l'applicazione TFFNL
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


class	TFFNL
{
public:
	TFFNL();
	int			main(int na, char **a);

private:

	static	BOOL __stdcall ctrlBreakHandler( DWORD dw_CtrlType );
	void 		usage();
	int 		getResp();
	int		htoi(char *s);
	void 		showError(const char* sz_filename);
	int 		findFile(char *psz_path);

	stringList	IncludeList;
	stringList	ExcludeList;

	int		i_maxElementLen;
	bool		b_recourseDirectory;
	bool		b_directoryOnly;
	bool		b_fileOnly;
	int		i_numOfMatch;
	int		i_separator;
};

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: TFFNL ctor
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

TFFNL::TFFNL()
:
	i_maxElementLen(-1),
	b_recourseDirectory(false),
	b_directoryOnly(false),
	b_fileOnly(false),
	i_numOfMatch(0),
	i_separator(0x09)
{
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Control-break handler
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

BOOL __stdcall TFFNL::ctrlBreakHandler( DWORD dw_CtrlType )
{
	if( dw_CtrlType == CTRL_C_EVENT || dw_CtrlType == CTRL_BREAK_EVENT )
	{
		cerr << ("Ctrl-Break pressed! Program aborted");
		ExitProcess(2);
		return TRUE;
	}
	else
		return FALSE; // Don't handle 'CLOSE', 'LOGOFF' or 'SHUTDOWN' events.
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: print usage msg to stderr when the args in command line is less than 2
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

void TFFNL::usage()
{
    cout
        << "\n"
        << "FFNL32 (Win32) 1.0 " __DATE__ "\n"
        << "Multi-pattern Find Filename & Len\n"
        << "Usage: FFNL32 [options] [d:][path]filename [&|!] [filename] [@filelist]...\n"
        << "options:\n"
        << "  -C[hh] set separator char (default: 0x09 (TAB), -1 to disable)\n"
        << "  -D search only directory name (default files & dir)\n"
        << "  -F search only filename (default files & dir)\n"
        << "  -M[n] set max element size (default -1)\n"
        << "  -R recourse subdirectories\n\n"
        << "  Press any key to continue.\n"
		<<	endl
	;

	while(!kbhit())
	;

	getch();

	cout
		<<	"Directory search use the following search criteria:\n"
			"* -- Matches 0 or more characters of any type\n"
			"@ -- a..z, A..Z\n"
			"# -- 0..9\n"
			"+ -- A..z, A..Z, 0..9\n"
			"? -- A-F, a-f, 0-9 (a hex number)\n"
			"< -- a..z\n"
			"> -- A..Z\n"
			"= -- Any one character\n"
			"\" -- Escape.  The next character in the pattern much match exactly.\n"
			"wild cards will not be interpreted.\n"
			"( -- Group.  A ""("" starts a group and a "")"" ends it.  For each\n"
			"group, there must be a matching character in the string.\n"
			"Example: (abc) matches b.  Wild cards are not interpreted.\n"
			"\\ -- The string to the right of the \\ is optional.  It can be\n"
			"present and if so must match.  If it is missing, it is a match.\n"
			"Example: a\\a will match with a or aa but not ab\n\n"

		<<	"File search use the normal *? wildcard search criteria."
        << endl
	;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Convert hex string to int
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

int TFFNL::htoi(char *s)
{
	int i = 0;
	char c;

	while((c = *s++) != '\0')
	{
		if(c == 'x')
			continue;   /* allow 0x notation */
		if('0' <= c && c <= '9')
			i = (i * 16) + (c - '0');
		else if('a' <= c && c <= 'f')
			i = (i * 16) + (c - 'a' + 10);
		else if('A' <= c && c <= 'F')
			i = (i * 16) + (c - 'A' + 10);
		else
			break;
	}
	return i;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Get resp from query
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

int TFFNL::getResp()
{
    char	c_rsp;

    do
    {
        c_rsp=char(toupper(getch()));
        if(c_rsp=='\x03') return ('Q');
    }
    while (c_rsp != 'Y' && c_rsp != 'N' && c_rsp != 'X' && c_rsp !='Q');

    putchar(c_rsp);
    putchar('\n');

    return(c_rsp);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Show the error message
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

void TFFNL::showError( const char* sz_filename )
{
	switch(errno)
	{
#ifdef	__BORLANDC__
		case ECURDIR:
			cout << "Attempted to remove current directory" << endl;
			break;
#endif	//	__BORLANDC__
		case ENODEV :
            cout	<< "Invalid device!" << endl;
			break;
#ifdef	__BORLANDC__
		case ENOFILE :
		case ENOPATH :
#elif	defined (_MSC_VER)
		case ENOENT:
#endif
			cout	<< "Invalid path or filename!" << endl;
			break;
		case EACCES :
			cout	<< "Permission denied!" << endl;
			break;
		default:
			cout	<< "I/O Error number:" << errno << endl;
			break;
	}

	cout << "Reading " << sz_filename << endl;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Esegue il filesearch
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


int TFFNL::findFile(char *psz_path)
{
	char 		sz_dirSearch[_MAX_PATH];
	char		sz_dirPath[_MAX_PATH];
	char		sz_findFile[_MAX_PATH];
	int			i_filenameLen;

	HANDLE            h_Search;
	WIN32_FIND_DATA   findData;

	strcpy(sz_dirSearch,psz_path);
	strcpy(sz_dirPath,sz_dirSearch);
	strcat(sz_dirSearch,"*.*");

   // Look for a file
	h_Search = FindFirstFile (sz_dirSearch, &findData);

	if (!h_Search || h_Search == (HANDLE) -1)
	{
		showError(sz_dirSearch);
		return true;
	}

	// for each file in the directory check if its name match the pattern
	// list

	do
	{
		bool 	b_deleteOk=false;
		bool	b_deleteNo=false;

		if (*findData.cFileName != '.')
		{
			strcpy(sz_findFile,sz_dirPath);
			strcat(sz_findFile,findData.cFileName);

			i_filenameLen	=	strlen(findData.cFileName);

			if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)    /* if the filename is a subdir then */
			{
         		if(!b_fileOnly)
				{
				   if (!IncludeList.empty())
				   {
#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
						stringListIterator	iter	=	find_if(IncludeList.begin(), IncludeList.end(), matchDirname(findData.cFileName));

						if(*iter)
							b_deleteOk=true;
#else
						stringListIterator iter	=	find_if(IncludeList.begin(), IncludeList.end(), matchDirname(findData.cFileName));

						if(iter != IncludeList.end())
							b_deleteOk=true;
#endif
				   }
				   else
						b_deleteOk=true;

				   if (!ExcludeList.empty())
				   {
#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
						stringListIterator	iter	=	find_if(ExcludeList.begin(), ExcludeList.end(),matchDirname(findData.cFileName));

						if(*iter)
							b_deleteNo=true;
#else
						list<string>::iterator iter	=	find_if(ExcludeList.begin(), ExcludeList.end(),matchDirname(findData.cFileName));

						if(iter != ExcludeList.end())
							b_deleteNo=true;
#endif
				   }

				   if (b_deleteOk && !b_deleteNo)
				   {
						if(i_maxElementLen!=-1)
						{
							if(i_filenameLen > i_maxElementLen)
							{
								i_numOfMatch++;

								if(i_separator==-1)
									cout << sz_dirPath << findData.cFileName << endl;
								else
									cout << sz_dirPath << char(i_separator) << findData.cFileName << endl;
							}
						}
						else
						{
							i_numOfMatch++;

                     if(i_separator==-1)
                        cout << sz_dirPath << findData.cFileName << endl;
                     else
                        cout << sz_dirPath << char(i_separator) << findData.cFileName << endl;
						}

				   }
				}

				if(b_recourseDirectory)
				{
					strcat(sz_findFile,"\\");

					if(findFile(sz_findFile))
						return(1);
				}
			}
			else
			{
         		if( !b_directoryOnly)
				{
				   if (!IncludeList.empty())
				   {
#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
						stringListIterator	iter	=	find_if(IncludeList.begin(), IncludeList.end(), matchFilename(findData.cFileName));

						if(*iter)
							b_deleteOk=true;
#else
						stringListIterator iter	=	find_if(IncludeList.begin(), IncludeList.end(), matchFilename(findData.cFileName));

						if(iter != IncludeList.end())
							b_deleteOk=true;
#endif
				   }
				   else
						b_deleteOk=true;

				   if (!ExcludeList.empty())
				   {
#if  defined __BORLANDC__ && 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53
						stringListIterator	iter	=	find_if(ExcludeList.begin(), ExcludeList.end(),matchFilename(findData.cFileName));

						if(*iter)
							b_deleteNo=true;
#else
						list<string>::iterator iter	=	find_if(ExcludeList.begin(), ExcludeList.end(),matchFilename(findData.cFileName));

						if(iter != ExcludeList.end())
							b_deleteNo=true;
#endif
				   }

				   if (b_deleteOk && !b_deleteNo)
				   {
						if(i_maxElementLen!=-1)
						{
							if(i_filenameLen > i_maxElementLen)
							{
								i_numOfMatch++;
								
								if(i_separator==-1)
									cout << sz_dirPath << findData.cFileName << endl;
								else
									cout << sz_dirPath << char(i_separator) << findData.cFileName << endl;
							}
						}
						else
						{
							i_numOfMatch++;

							if(i_separator==-1)
								cout << sz_dirPath << findData.cFileName << endl;
							else
								cout << sz_dirPath << char(i_separator) << findData.cFileName << endl;
						}
				   }
				}
			}
		}
	}
	while(FindNextFile (h_Search, &findData));

	FindClose(h_Search);

	return (0);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: The main thing
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

int TFFNL::main (int na, char** a)
{
	char	sz_path[_MAX_PATH];
	char	sz_drive[_MAX_DRIVE];
	char	sz_dir[_MAX_DIR];
	char	sz_name[_MAX_FNAME];
	char	sz_ext[_MAX_EXT];
	char	sz_currentPath[_MAX_PATH];
	bool 	b_AND=true;
	bool	b_firstArg=true;
	bool	b_selectInclude=false;

	// check the number of args
	// if less of 2 then print usage message

	if (na < 2)
	{
		usage();
		return -1;
	}

	SetConsoleCtrlHandler( &TFFNL::ctrlBreakHandler, TRUE );

	memset(sz_currentPath, 0, _MAX_PATH);

   // parse command line parameter and fill the 2 simple list:

   // Ok_List contens the match pattern
   // Ko_List contens the don't match pattern

	for(int i=1; i<na; i++)
	{
  		char* p=a[i];

		if(*p == '&')
		{
      		if(*(p+1) == NULL)
			{
         		if(!b_firstArg)
					b_AND=true;
				else
				{
					b_firstArg=false;
#ifdef	__BORLANDC__
					getcwd (sz_currentPath, sizeof(sz_currentPath));
#else
					_getcwd (sz_currentPath, sizeof(sz_currentPath));
#endif
					strcat (sz_currentPath,"\\");
				}
				continue;
			}
			else
				b_selectInclude=true;
		}

		if(*p == '!')
		{
			if(*(p+1) == NULL)
			{
				b_AND=false;
				if(b_firstArg)
				{
					b_firstArg=false;
#ifdef	__BORLANDC__
					getcwd (sz_currentPath, sizeof(sz_currentPath));
#else
					_getcwd (sz_currentPath, sizeof(sz_currentPath));
#endif
					strcat (sz_currentPath,"\\");
				}
				continue;
			}
			else
				b_selectInclude=true;
		}

		if (*p == '@')
		{
			if(*(++p) != NULL)
			{
				if(b_firstArg)
					b_firstArg=false;

				ifstream listfile(p);

				if(!listfile)
				{
					cerr << "Error opening List File " << p << endl;
					return 1;
				}

				char sz_filename[_MAX_PATH];

				while(listfile && listfile >> sz_filename)
				{
            		string *argstr = new string(sz_filename);

					if (b_AND)
               			IncludeList.push_front(*argstr);
					else
               			ExcludeList.push_front(*argstr);
				}
				continue;
			}
		}

		if(b_firstArg && (*p == '-' || *p == '/' ))
		{
      	switch (toupper(*(p+1)))
			{
				case 'C':
					++p;
					if(*(p+1))
               {
               	if(*(p+1) == '-')
                  {
                  	++p;
	                  i_separator	=	char(htoi(++p));
                     i_separator	*= -1;
                  }
                  else
            			i_separator		=	char(htoi(++p));
               }
				   break;
            case 'D':
            		b_directoryOnly=true;
				   break;

            case 'F':
            	b_fileOnly=true;
				   break;

				case 'M':
					++p;
					if(*(p+1))
						i_maxElementLen=atoi(++p);
				   break;

				case 'R':
            		b_recourseDirectory=true;
						break;

				case '?':
            		usage();
						break;
				
				default:
            		break;
			}
			continue;
		}

		if(b_selectInclude)
		{
			b_selectInclude=false;
			strrev(a[i]);
			a[i][strlen(a[i]-2)]=NULL;
			strrev(a[i]);
		}

		if (b_firstArg)
		{
      		b_firstArg=false;
			strcpy(sz_currentPath,a[i]);
			continue;
		}

		string*	argstr = new string(a[i]);

		if (b_AND)
			IncludeList.push_front(*argstr);
		else
			ExcludeList.push_front(*argstr);
	}

	strupr(sz_currentPath);
	_splitpath(sz_currentPath,sz_drive,sz_dir,sz_name,sz_ext);

	if (*sz_name || *sz_ext)
	{
		strcpy(sz_path,sz_name);
		strcat(sz_path,sz_ext);

		IncludeList.push_front(*new string(sz_path));
	}

//	if(IncludeList.empty())
//		IncludeList.push_front(*new string("*.*"));

	strcpy(sz_path,sz_drive);
	strcat(sz_path,sz_dir);

	int ret=findFile(sz_path);

	cout
   	 	<<	endl
			<< "Total found : " << setw(6) << i_numOfMatch
			<< endl;

	SetConsoleCtrlHandler( NULL, FALSE);

	return (ret);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco						Data di creazione : 19/7/99
 * Codice   :
 *
 * Note: Program entry point
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 --------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

int main (int na, char** a)
{
	return	TFFNL().main(na, a);
}

